@@ -3,6 +3,7 @@
  * GData Client
  * Copyright (C) Thibault Saunier 2009 <saunierthibault@gmail.com>
  * Copyright (C) Philip Withnall 2010 <philip@tecnocode.co.uk>
+ * Copyright (C) Red Hat, Inc. 2015, 2016
  *
  * GData Client is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,9 +30,32 @@
 
 #include "gdata.h"
 #include "common.h"
+#include "gdata-dummy-authorizer.h"
 
 static UhmServer *mock_server = NULL;
 
+#undef CLIENT_ID  /* from common.h */
+
+#define CLIENT_ID "352818697630-nqu2cmt5quqd6lr17ouoqmb684u84l1f.apps.googleusercontent.com"
+#define CLIENT_SECRET "-fA4pHQJxR3zJ-FyAMPQsikg"
+#define REDIRECT_URI "urn:ietf:wg:oauth:2.0:oob"
+
+static void
+add_folder_link_to_entry (GDataDocumentsEntry *entry, GDataDocumentsFolder *folder)
+{
+	GDataLink *_link;
+	const gchar *id;
+	gchar *uri;
+
+	/* HACK: Build the GDataLink:uri from the ID by adding the prefix. */
+	id = gdata_entry_get_id (GDATA_ENTRY (folder));
+	uri = g_strconcat ("https://www.googleapis.com/drive/v2/files/", id, NULL);
+	_link = gdata_link_new (uri, GDATA_LINK_PARENT);
+	gdata_entry_add_link (GDATA_ENTRY (entry), _link);
+	g_object_unref (_link);
+	g_free (uri);
+}
+
 static gboolean
 check_document_is_in_folder (GDataDocumentsDocument *document, GDataDocumentsFolder *folder)
 {
@@ -46,8 +70,7 @@ check_document_is_in_folder (GDataDocumentsDocument *document, GDataDocumentsFol
 	     links != NULL; links = links->next) {
 		GDataLink *_link = GDATA_LINK (links->data);
 
-		if (strcmp (gdata_link_get_uri (_link), gdata_link_get_uri (folder_self_link)) == 0 &&
-		    strcmp (gdata_link_get_title (_link), gdata_entry_get_title (GDATA_ENTRY (folder))) == 0) {
+		if (strcmp (gdata_link_get_uri (_link), gdata_link_get_uri (folder_self_link)) == 0) {
 			g_assert (found_folder_category == FALSE);
 			found_folder_category = TRUE;
 		}
@@ -89,19 +112,28 @@ static GDataDocumentsFolder *
 create_folder (GDataDocumentsService *service, const gchar *title)
 {
 	GDataDocumentsFolder *folder, *new_folder;
-	gchar *upload_uri;
+	GDataDocumentsFolder *root;
+
+	root = GDATA_DOCUMENTS_FOLDER (gdata_service_query_single_entry (GDATA_SERVICE (service),
+	                                                                 gdata_documents_service_get_primary_authorization_domain (),
+	                                                                 "root",
+	                                                                 NULL,
+	                                                                 GDATA_TYPE_DOCUMENTS_FOLDER,
+	                                                                 NULL,
+	                                                                 NULL));
 
 	folder = gdata_documents_folder_new (NULL);
 	gdata_entry_set_title (GDATA_ENTRY (folder), title);
 
 	/* Insert the folder */
-	upload_uri = gdata_documents_service_get_upload_uri (NULL);
-	new_folder = GDATA_DOCUMENTS_FOLDER (gdata_service_insert_entry (GDATA_SERVICE (service),
-	                                                                 gdata_documents_service_get_primary_authorization_domain (),
-	                                                                 upload_uri, GDATA_ENTRY (folder), NULL, NULL));
+	new_folder = GDATA_DOCUMENTS_FOLDER (gdata_documents_service_add_entry_to_folder (GDATA_DOCUMENTS_SERVICE (service),
+	                                                                                  GDATA_DOCUMENTS_ENTRY (folder),
+	                                                                                  root,
+	                                                                                  NULL,
+	                                                                                  NULL));
 	g_assert (GDATA_IS_DOCUMENTS_FOLDER (new_folder));
-	g_free (upload_uri);
 	g_object_unref (folder);
+	g_object_unref (root);
 
 	return new_folder;
 }
@@ -109,81 +141,48 @@ create_folder (GDataDocumentsService *service, const gchar *title)
 static void
 test_authentication (void)
 {
-	gboolean retval;
-	GDataClientLoginAuthorizer *authorizer;
-	GError *error = NULL;
+	GDataOAuth2Authorizer *authorizer = NULL;  /* owned */
+	gchar *authentication_uri, *authorisation_code;
 
 	gdata_test_mock_server_start_trace (mock_server, "authentication");
 
-	/* Create an authorizer */
-	authorizer = gdata_client_login_authorizer_new (CLIENT_ID, GDATA_TYPE_DOCUMENTS_SERVICE);
+	authorizer = gdata_oauth2_authorizer_new (CLIENT_ID, CLIENT_SECRET, REDIRECT_URI, GDATA_TYPE_DOCUMENTS_SERVICE);
 
-	g_assert_cmpstr (gdata_client_login_authorizer_get_client_id (authorizer), ==, CLIENT_ID);
+	/* Get an authentication URI. */
+	authentication_uri = gdata_oauth2_authorizer_build_authentication_uri (authorizer, NULL, FALSE);
+	g_assert (authentication_uri != NULL);
 
-	/* Log in */
-	retval = gdata_client_login_authorizer_authenticate (authorizer, USERNAME, PASSWORD, NULL, &error);
-	g_assert_no_error (error);
-	g_assert (retval == TRUE);
-	g_clear_error (&error);
+	/* Get the authorisation code off the user. */
+	if (uhm_server_get_enable_online (mock_server)) {
+		authorisation_code = gdata_test_query_user_for_verifier (authentication_uri);
+	} else {
+		/* Hard coded, extracted from the trace file. */
+		authorisation_code = g_strdup ("FIXME: to be updated from the trace file");
+	}
+
+	g_free (authentication_uri);
+
+	if (authorisation_code == NULL) {
+		/* Skip tests. */
+		goto skip_test;
+	}
+
+	/* Authorise the token */
+	g_assert (gdata_oauth2_authorizer_request_authorization (authorizer, authorisation_code, NULL, NULL) == TRUE);
 
 	/* Check all is as it should be */
-	g_assert_cmpstr (gdata_client_login_authorizer_get_username (authorizer), ==, USERNAME);
-	g_assert_cmpstr (gdata_client_login_authorizer_get_password (authorizer), ==, PASSWORD);
-
 	g_assert (gdata_authorizer_is_authorized_for_domain (GDATA_AUTHORIZER (authorizer),
 	                                                     gdata_documents_service_get_primary_authorization_domain ()) == TRUE);
 	g_assert (gdata_authorizer_is_authorized_for_domain (GDATA_AUTHORIZER (authorizer),
 	                                                     gdata_documents_service_get_spreadsheet_authorization_domain ()) == TRUE);
 
+skip_test:
+	g_free (authorisation_code);
 	g_object_unref (authorizer);
 
 	uhm_server_end_trace (mock_server);
 }
 
-GDATA_ASYNC_TEST_FUNCTIONS (authentication, void,
-G_STMT_START {
-	GDataClientLoginAuthorizer *authorizer;
-
-	/* Create an authorizer */
-	authorizer = gdata_client_login_authorizer_new (CLIENT_ID, GDATA_TYPE_DOCUMENTS_SERVICE);
-
-	g_assert_cmpstr (gdata_client_login_authorizer_get_client_id (authorizer), ==, CLIENT_ID);
-
-	gdata_client_login_authorizer_authenticate_async (authorizer, USERNAME, PASSWORD, cancellable, async_ready_callback, async_data);
-
-	g_object_unref (authorizer);
-} G_STMT_END,
-G_STMT_START {
-	gboolean retval;
-	GDataClientLoginAuthorizer *authorizer = GDATA_CLIENT_LOGIN_AUTHORIZER (obj);
-
-	retval = gdata_client_login_authorizer_authenticate_finish (authorizer, async_result, &error);
-
-	if (error == NULL) {
-		g_assert (retval == TRUE);
-
-		/* Check all is as it should be */
-		g_assert_cmpstr (gdata_client_login_authorizer_get_username (authorizer), ==, USERNAME);
-		g_assert_cmpstr (gdata_client_login_authorizer_get_password (authorizer), ==, PASSWORD);
-
-		g_assert (gdata_authorizer_is_authorized_for_domain (GDATA_AUTHORIZER (authorizer),
-		                                                     gdata_documents_service_get_primary_authorization_domain ()) == TRUE);
-		g_assert (gdata_authorizer_is_authorized_for_domain (GDATA_AUTHORIZER (authorizer),
-		                                                     gdata_documents_service_get_spreadsheet_authorization_domain ()) == TRUE);
-	} else {
-		g_assert (retval == FALSE);
-
-		/* Check nothing's changed */
-		g_assert_cmpstr (gdata_client_login_authorizer_get_username (authorizer), ==, NULL);
-		g_assert_cmpstr (gdata_client_login_authorizer_get_password (authorizer), ==, NULL);
-
-		g_assert (gdata_authorizer_is_authorized_for_domain (GDATA_AUTHORIZER (authorizer),
-		                                                     gdata_documents_service_get_primary_authorization_domain ()) == FALSE);
-		g_assert (gdata_authorizer_is_authorized_for_domain (GDATA_AUTHORIZER (authorizer),
-		                                                     gdata_documents_service_get_spreadsheet_authorization_domain ()) == FALSE);
-	}
-} G_STMT_END);
-
 typedef struct {
 	GDataDocumentsFolder *folder;
 } TempFolderData;
@@ -192,22 +191,31 @@ static void
 set_up_temp_folder (TempFolderData *data, gconstpointer service)
 {
 	GDataDocumentsFolder *folder;
-	gchar *upload_uri;
+	GDataDocumentsFolder *root;
 
 	gdata_test_mock_server_start_trace (mock_server, "setup-temp-folder");
 
+	root = GDATA_DOCUMENTS_FOLDER (gdata_service_query_single_entry (GDATA_SERVICE (service),
+	                                                                 gdata_documents_service_get_primary_authorization_domain (),
+	                                                                 "root",
+	                                                                 NULL,
+	                                                                 GDATA_TYPE_DOCUMENTS_FOLDER,
+	                                                                 NULL,
+	                                                                 NULL));
+
 	/* Create a folder */
 	folder = gdata_documents_folder_new (NULL);
 	gdata_entry_set_title (GDATA_ENTRY (folder), "Temporary Folder");
 
 	/* Insert the folder */
-	upload_uri = gdata_documents_service_get_upload_uri (NULL);
-	data->folder = GDATA_DOCUMENTS_FOLDER (gdata_service_insert_entry (GDATA_SERVICE (service),
-	                                                                   gdata_documents_service_get_primary_authorization_domain (),
-	                                                                   upload_uri, GDATA_ENTRY (folder), NULL, NULL));
+	data->folder = GDATA_DOCUMENTS_FOLDER (gdata_documents_service_add_entry_to_folder (GDATA_DOCUMENTS_SERVICE (service),
+	                                                                                    GDATA_DOCUMENTS_ENTRY (folder),
+	                                                                                    root,
+	                                                                                    NULL,
+	                                                                                    NULL));
 	g_assert (GDATA_IS_DOCUMENTS_FOLDER (data->folder));
-	g_free (upload_uri);
 	g_object_unref (folder);
+	g_object_unref (root);
 
 	uhm_server_end_trace (mock_server);
 }
@@ -240,15 +248,16 @@ _set_up_temp_document (GDataDocumentsEntry *entry, GDataService *service, GFile
 
 	/* Query for information on the file. */
 	file_info = g_file_query_info (document_file,
-	                               G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME "," G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
-	                               G_FILE_ATTRIBUTE_STANDARD_SIZE, G_FILE_QUERY_INFO_NONE, NULL, &error);
+	                               G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME "," G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+	                               G_FILE_QUERY_INFO_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	/* Prepare the upload stream */
-	upload_stream = gdata_documents_service_upload_document_resumable (GDATA_DOCUMENTS_SERVICE (service), GDATA_DOCUMENTS_DOCUMENT (entry),
-	                                                                   g_file_info_get_display_name (file_info),
-	                                                                   g_file_info_get_content_type (file_info), g_file_info_get_size (file_info),
-	                                                                   NULL, NULL, &error);
+	upload_stream = gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service),
+	                                                         GDATA_DOCUMENTS_DOCUMENT (entry),
+	                                                         g_file_info_get_display_name (file_info),
+	                                                         g_file_info_get_content_type (file_info),
+	                                                         NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_UPLOAD_STREAM (upload_stream));
 
@@ -300,7 +309,7 @@ set_up_temp_document_spreadsheet (TempDocumentData *data, gconstpointer service)
 	document = gdata_documents_spreadsheet_new (NULL);
 	gdata_entry_set_title (GDATA_ENTRY (document), "Temporary Document (Spreadsheet)");
 
-	document_file_path = g_strconcat (TEST_FILE_DIR, "test.ods", NULL);
+	document_file_path = g_test_build_filename (G_TEST_DIST, "test.ods", NULL);
 	document_file = g_file_new_for_path (document_file_path);
 	g_free (document_file_path);
 
@@ -347,13 +356,10 @@ test_delete_folder (TempFolderData *data, gconstpointer service)
 	updated_folder = gdata_service_query_single_entry (GDATA_SERVICE (service), gdata_documents_service_get_primary_authorization_domain (),
 	                                                   gdata_entry_get_id (GDATA_ENTRY (data->folder)), NULL,
 	                                                   GDATA_TYPE_DOCUMENTS_FOLDER, NULL, &error);
-	g_assert_no_error (error);
-	g_assert (GDATA_IS_DOCUMENTS_FOLDER (updated_folder));
+	g_assert_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_NOT_FOUND);
+	g_assert (updated_folder == NULL);
 	g_clear_error (&error);
 
-	g_assert (gdata_documents_entry_is_deleted (GDATA_DOCUMENTS_ENTRY (updated_folder)) == TRUE);
-
-	g_object_unref (updated_folder);
 	g_object_unref (data->folder);
 	data->folder = NULL;
 
@@ -382,13 +388,10 @@ test_delete_document (TempDocumentData *data, gconstpointer service)
 	updated_document = gdata_service_query_single_entry (GDATA_SERVICE (service), gdata_documents_service_get_primary_authorization_domain (),
 	                                                     gdata_entry_get_id (GDATA_ENTRY (data->document)), NULL,
 	                                                     G_OBJECT_TYPE (data->document), NULL, &error);
-	g_assert_no_error (error);
-	g_assert (GDATA_IS_DOCUMENTS_DOCUMENT (updated_document));
+	g_assert_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_NOT_FOUND);
+	g_assert (updated_document == NULL);
 	g_clear_error (&error);
 
-	g_assert (gdata_documents_entry_is_deleted (GDATA_DOCUMENTS_ENTRY (updated_document)) == TRUE);
-
-	g_object_unref (updated_document);
 	g_object_unref (data->document);
 	data->document = NULL;
 
@@ -407,11 +410,9 @@ static void
 set_up_temp_documents (TempDocumentsData *data, gconstpointer service)
 {
 	GDataDocumentsEntry *document;
-	gchar *upload_uri, *document_file_path;
+	gchar *document_file_path;
 	GFile *document_file;
 
-	upload_uri = gdata_documents_service_get_upload_uri (NULL);
-
 	/* Create a temporary folder */
 	set_up_temp_folder ((TempFolderData*) data, service);
 
@@ -421,8 +422,8 @@ set_up_temp_documents (TempDocumentsData *data, gconstpointer service)
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_spreadsheet_new (NULL));
 	gdata_entry_set_title (GDATA_ENTRY (document), "Temporary Spreadsheet");
 	data->spreadsheet_document = GDATA_DOCUMENTS_SPREADSHEET (
-		gdata_service_insert_entry (GDATA_SERVICE (service), gdata_documents_service_get_primary_authorization_domain (),
-		                            upload_uri, GDATA_ENTRY (document), NULL, NULL)
+		gdata_documents_service_add_entry_to_folder (GDATA_DOCUMENTS_SERVICE (service), document, ((TempFolderData*) data)->folder,
+		                                             NULL, NULL)
 	);
 	g_assert (GDATA_IS_DOCUMENTS_SPREADSHEET (data->spreadsheet_document));
 	g_object_unref (document);
@@ -430,8 +431,8 @@ set_up_temp_documents (TempDocumentsData *data, gconstpointer service)
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_presentation_new (NULL));
 	gdata_entry_set_title (GDATA_ENTRY (document), "Temporary Presentation");
 	data->presentation_document = GDATA_DOCUMENTS_PRESENTATION (
-		gdata_service_insert_entry (GDATA_SERVICE (service), gdata_documents_service_get_primary_authorization_domain (),
-		                            upload_uri, GDATA_ENTRY (document), NULL, NULL)
+		gdata_documents_service_add_entry_to_folder (GDATA_DOCUMENTS_SERVICE (service), document, ((TempFolderData*) data)->folder,
+		                                             NULL, NULL)
 	);
 	g_assert (GDATA_IS_DOCUMENTS_PRESENTATION (data->presentation_document));
 	g_object_unref (document);
@@ -439,13 +440,13 @@ set_up_temp_documents (TempDocumentsData *data, gconstpointer service)
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_text_new (NULL));
 	gdata_entry_set_title (GDATA_ENTRY (document), "Temporary Text Document");
 	data->text_document = GDATA_DOCUMENTS_TEXT (
-		gdata_service_insert_entry (GDATA_SERVICE (service), gdata_documents_service_get_primary_authorization_domain (),
-		                            upload_uri, GDATA_ENTRY (document), NULL, NULL)
+		gdata_documents_service_add_entry_to_folder (GDATA_DOCUMENTS_SERVICE (service), document, ((TempFolderData*) data)->folder,
+		                                             NULL, NULL)
 	);
 	g_assert (GDATA_IS_DOCUMENTS_TEXT (data->text_document));
 	g_object_unref (document);
 
-	document_file_path = g_strconcat (TEST_FILE_DIR, "test.odt", NULL);
+	document_file_path = g_test_build_filename (G_TEST_DIST, "test.odt", NULL);
 	document_file = g_file_new_for_path (document_file_path);
 	g_free (document_file_path);
 
@@ -456,7 +457,6 @@ set_up_temp_documents (TempDocumentsData *data, gconstpointer service)
 	g_object_unref (document);
 
 	g_object_unref (document_file);
-	g_free (upload_uri);
 
 	uhm_server_end_trace (mock_server);
 }
@@ -768,7 +768,7 @@ test_upload (UploadDocumentData *data, gconstpointer _test_params)
 			break;
 		case UPLOAD_CONTENT_ONLY:
 		case UPLOAD_CONTENT_AND_METADATA: {
-			gchar *document_file_path = g_strconcat (TEST_FILE_DIR, document_filename, NULL);
+			gchar *document_file_path = g_test_build_filename (G_TEST_DIST, document_filename, NULL);
 			document_file = g_file_new_for_path (document_file_path);
 			g_free (document_file_path);
 
@@ -816,15 +816,9 @@ test_upload (UploadDocumentData *data, gconstpointer _test_params)
 	}
 
 	if (test_params->payload_type == UPLOAD_METADATA_ONLY) {
-		gchar *upload_uri;
-
-		/* Insert the document */
-		upload_uri = gdata_documents_service_get_upload_uri (data->folder);
-		data->new_document = GDATA_DOCUMENTS_DOCUMENT (gdata_service_insert_entry (GDATA_SERVICE (test_params->service),
-		                                                                           gdata_documents_service_get_primary_authorization_domain (),
-		                                                                           upload_uri, GDATA_ENTRY (document), NULL, &error));
-		g_free (upload_uri);
-
+		data->new_document = GDATA_DOCUMENTS_DOCUMENT (gdata_documents_service_add_entry_to_folder (test_params->service,
+		                                                                                            GDATA_DOCUMENTS_ENTRY (document),
+		                                                                                            data->folder, NULL, &error));
 		g_assert_no_error (error);
 	} else {
 		GDataUploadStream *upload_stream;
@@ -976,7 +970,7 @@ set_up_update_document (UpdateDocumentData *data, gconstpointer _test_params)
 	gdata_entry_set_title (GDATA_ENTRY (document), title);
 	g_free (title);
 
-	document_file_path = g_strconcat (TEST_FILE_DIR, "test.odt", NULL);
+	document_file_path = g_test_build_filename (G_TEST_DIST, "test.odt", NULL);
 	document_file = g_file_new_for_path (document_file_path);
 	g_free (document_file_path);
 
@@ -1064,9 +1058,12 @@ test_update (UpdateDocumentData *data, gconstpointer _test_params)
 		GFileInputStream *file_stream;
 		GFile *updated_document_file;
 		GFileInfo *file_info;
+		gchar *path = NULL;
 
 		/* Prepare the updated file */
-		updated_document_file = g_file_new_for_path (TEST_FILE_DIR "test_updated.odt");
+		path = g_test_build_filename (G_TEST_DIST, "test_updated.odt", NULL);
+		updated_document_file = g_file_new_for_path (path);
+		g_free (path);
 
 		file_info = g_file_query_info (updated_document_file, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
 		                               G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE "," G_FILE_ATTRIBUTE_STANDARD_SIZE, G_FILE_QUERY_INFO_NONE,
@@ -1194,34 +1191,48 @@ static void
 set_up_folders (FoldersData *data, GDataDocumentsService *service, gboolean initially_in_folder)
 {
 	GDataDocumentsFolder *folder;
+	GDataDocumentsFolder *root;
 	GDataDocumentsDocument *document, *new_document;
 	GDataUploadStream *upload_stream;
 	GFileInputStream *file_stream;
 	GFile *document_file;
 	GFileInfo *file_info;
-	gchar *upload_uri;
+	gchar *path = NULL;
 	GError *error = NULL;
 
+	root = GDATA_DOCUMENTS_FOLDER (gdata_service_query_single_entry (GDATA_SERVICE (service),
+	                                                                 gdata_documents_service_get_primary_authorization_domain (),
+	                                                                 "root",
+	                                                                 NULL,
+	                                                                 GDATA_TYPE_DOCUMENTS_FOLDER,
+	                                                                 NULL,
+	                                                                 NULL));
 	/* Create a new folder for the tests */
 	folder = gdata_documents_folder_new (NULL);
 	gdata_entry_set_title (GDATA_ENTRY (folder), "add_file_folder_move_folder");
 
 	/* Insert the folder */
-	upload_uri = gdata_documents_service_get_upload_uri (NULL);
-	data->folder = GDATA_DOCUMENTS_FOLDER (gdata_service_insert_entry (GDATA_SERVICE (service),
-	                                                                   gdata_documents_service_get_primary_authorization_domain (),
-	                                                                   upload_uri, GDATA_ENTRY (folder), NULL, &error));
-	g_free (upload_uri);
-
+	data->folder = GDATA_DOCUMENTS_FOLDER (gdata_documents_service_add_entry_to_folder (GDATA_DOCUMENTS_SERVICE (service),
+	                                                                                    GDATA_DOCUMENTS_ENTRY (folder),
+	                                                                                    root,
+	                                                                                    NULL,
+	                                                                                    &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_DOCUMENTS_FOLDER (data->folder));
 
 	g_object_unref (folder);
 
 	/* Create a new file for the tests */
-	document_file = g_file_new_for_path (TEST_FILE_DIR "test.odt");
+	path = g_test_build_filename (G_TEST_DIST, "test.odt", NULL);
+	document_file = g_file_new_for_path (path);
+	g_free (path);
+
 	document = GDATA_DOCUMENTS_DOCUMENT (gdata_documents_text_new (NULL));
 	gdata_entry_set_title (GDATA_ENTRY (document), "add_file_folder_move_text");
+	if (initially_in_folder)
+		add_folder_link_to_entry (GDATA_DOCUMENTS_ENTRY (document), root);
+
+	g_object_unref (root);
 
 	file_info = g_file_query_info (document_file, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME "," G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
 	                               G_FILE_QUERY_INFO_NONE, NULL, &error);
@@ -1588,27 +1599,27 @@ test_download_thumbnail (TempDocumentData *data, gconstpointer service)
 static void
 test_access_rule_insert (TempDocumentData *data, gconstpointer service)
 {
-	GDataAccessRule *access_rule, *new_access_rule;
+	GDataDocumentsAccessRule *access_rule, *new_access_rule;
 	GDataLink *_link;
 	GError *error = NULL;
 
 	gdata_test_mock_server_start_trace (mock_server, "access-rule-insert");
 
 	/* New access rule */
-	access_rule = gdata_access_rule_new (NULL);
-	gdata_access_rule_set_role (access_rule, GDATA_DOCUMENTS_ACCESS_ROLE_WRITER);
-	gdata_access_rule_set_scope (access_rule, GDATA_ACCESS_SCOPE_USER, "libgdata.test@gmail.com");
+	access_rule = gdata_documents_access_rule_new (NULL);
+	gdata_access_rule_set_role (GDATA_ACCESS_RULE (access_rule), GDATA_DOCUMENTS_ACCESS_ROLE_WRITER);
+	gdata_access_rule_set_scope (GDATA_ACCESS_RULE (access_rule), GDATA_ACCESS_SCOPE_USER, "libgdata.test@gmail.com");
 
 	/* Set access rules */
 	_link = gdata_entry_look_up_link (GDATA_ENTRY (data->document), GDATA_LINK_ACCESS_CONTROL_LIST);
 	g_assert (_link != NULL);
 
-	new_access_rule = GDATA_ACCESS_RULE (gdata_service_insert_entry (GDATA_SERVICE (service),
-	                                                                 gdata_documents_service_get_primary_authorization_domain (),
-	                                                                 gdata_link_get_uri (_link),
-	                                                                 GDATA_ENTRY (access_rule), NULL, &error));
+	new_access_rule = GDATA_DOCUMENTS_ACCESS_RULE (gdata_service_insert_entry (GDATA_SERVICE (service),
+										   gdata_documents_service_get_primary_authorization_domain (),
+										   gdata_link_get_uri (_link),
+										   GDATA_ENTRY (access_rule), NULL, &error));
 	g_assert_no_error (error);
-	g_assert (GDATA_IS_ACCESS_RULE (new_access_rule));
+	g_assert (GDATA_IS_DOCUMENTS_ACCESS_RULE (new_access_rule));
 	g_clear_error (&error);
 
 	/* TODO: Check if everything is as it should be */
@@ -2043,6 +2054,55 @@ mock_server_notify_resolver_cb (GObject *object, GParamSpec *pspec, gpointer use
 	}
 }
 
+/* Set up a global GDataAuthorizer to be used for all the tests. Unfortunately,
+ * the Google Drive API is effectively limited to OAuth2 authorisation,
+ * so this requires user interaction when online.
+ *
+ * If not online, use a dummy authoriser. */
+static GDataAuthorizer *
+create_global_authorizer (void)
+{
+	GDataOAuth2Authorizer *authorizer = NULL;  /* owned */
+	gchar *authentication_uri, *authorisation_code;
+	GError *error = NULL;
+
+	/* If not online, just return a dummy authoriser. */
+	if (!uhm_server_get_enable_online (mock_server)) {
+		return GDATA_AUTHORIZER (gdata_dummy_authorizer_new (GDATA_TYPE_DOCUMENTS_SERVICE));
+	}
+
+	/* Otherwise, go through the interactive OAuth dance. */
+	gdata_test_mock_server_start_trace (mock_server, "global-authentication");
+	authorizer = gdata_oauth2_authorizer_new (CLIENT_ID, CLIENT_SECRET, REDIRECT_URI, GDATA_TYPE_DOCUMENTS_SERVICE);
+
+	/* Get an authentication URI */
+	authentication_uri = gdata_oauth2_authorizer_build_authentication_uri (authorizer, NULL, FALSE);
+	g_assert (authentication_uri != NULL);
+
+	/* Get the authorisation code off the user. */
+	authorisation_code = gdata_test_query_user_for_verifier (authentication_uri);
+
+	g_free (authentication_uri);
+
+	if (authorisation_code == NULL) {
+		/* Skip tests. */
+		g_object_unref (authorizer);
+		authorizer = NULL;
+		goto skip_test;
+	}
+
+	/* Authorise the token */
+	g_assert (gdata_oauth2_authorizer_request_authorization (authorizer, authorisation_code, NULL, &error));
+	g_assert_no_error (error);
+
+skip_test:
+	g_free (authorisation_code);
+
+	uhm_server_end_trace (mock_server);
+
+	return GDATA_AUTHORIZER (authorizer);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -2050,27 +2110,22 @@ main (int argc, char *argv[])
 	GDataAuthorizer *authorizer = NULL;
 	GDataService *service = NULL;
 	GFile *trace_directory;
+	gchar *path = NULL;
 
 	gdata_test_init (argc, argv);
 
 	mock_server = gdata_test_get_mock_server ();
 	g_signal_connect (G_OBJECT (mock_server), "notify::resolver", (GCallback) mock_server_notify_resolver_cb, NULL);
-	trace_directory = g_file_new_for_path (TEST_FILE_DIR "traces/documents");
+	path = g_test_build_filename (G_TEST_DIST, "traces/documents", NULL);
+	trace_directory = g_file_new_for_path (path);
+	g_free (path);
 	uhm_server_set_trace_directory (mock_server, trace_directory);
 	g_object_unref (trace_directory);
 
-	gdata_test_mock_server_start_trace (mock_server, "global-authentication");
-	authorizer = GDATA_AUTHORIZER (gdata_client_login_authorizer_new (CLIENT_ID, GDATA_TYPE_DOCUMENTS_SERVICE));
-	gdata_client_login_authorizer_authenticate (GDATA_CLIENT_LOGIN_AUTHORIZER (authorizer), DOCUMENTS_USERNAME, PASSWORD, NULL, NULL);
-	uhm_server_end_trace (mock_server);
-
+	authorizer = create_global_authorizer ();
 	service = GDATA_SERVICE (gdata_documents_service_new (authorizer));
 
 	g_test_add_func ("/documents/authentication", test_authentication);
-	g_test_add ("/documents/authentication/async", GDataAsyncTestData, NULL, gdata_set_up_async_test_data, test_authentication_async,
-	            gdata_tear_down_async_test_data);
-	g_test_add ("/documents/authentication/async/cancellation", GDataAsyncTestData, NULL, gdata_set_up_async_test_data,
-	            test_authentication_async_cancellation, gdata_tear_down_async_test_data);
 
 	g_test_add ("/documents/delete/document", TempDocumentData, service, set_up_temp_document_spreadsheet, test_delete_document,
 	            tear_down_temp_document);
@@ -2087,10 +2142,20 @@ main (int argc, char *argv[])
 			for (j = 0; j < UPLOAD_FOLDER_TYPE_MAX + 1; j++) {
 				for (k = 0; k < UPLOAD_RESUMABLE_TYPE_MAX + 1; k++) {
 					for (l = 0; l < UPLOAD_DOCUMENT_TYPE_MAX + 1; l++) {
-					UploadDocumentTestParams *test_params;
-					gchar *test_name;
+						UploadDocumentTestParams *test_params;
+						gchar *test_name;
 
-					/* Resumable metadata-only uploads don't make sense. */
+						/* FIXME: Resumable uploads are not implemented. */
+						if (k == UPLOAD_RESUMABLE) {
+							continue;
+						}
+
+						/* FIXME: Conversion during uploads is not implemented. */
+						if (l == UPLOAD_ODT_CONVERT) {
+							continue;
+						}
+
+						/* Resumable metadata-only uploads don't make sense. */
 						if (i == UPLOAD_METADATA_ONLY && k == UPLOAD_RESUMABLE) {
 							continue;
 						}
@@ -2142,6 +2207,11 @@ main (int argc, char *argv[])
 					continue;
 				}
 
+				/* FIXME: Resumable uploads are not implemented. */
+				if (j == UPLOAD_RESUMABLE) {
+					continue;
+				}
+
 				test_name = g_strdup_printf ("/documents/update/%s/%s", payload_type_names[i], resumable_type_names[j]);
 
 				/* Allocate a new struct. We leak this. */
@@ -2208,5 +2278,7 @@ main (int argc, char *argv[])
 	if (service != NULL)
 		g_object_unref (service);
 
+	g_clear_object (&authorizer);
+
 	return retval;
 }
